from __future__ import absolute_import

from ggb.utils.image import GGBImage
from ggb.utils.constant import ColorSpace, CVLib
from ggb.utils.error import ColorSpaceError


import numpy as np


class GGB(GGBImage):
    """GGB color space converter.
    
    :param image: image source either from path or variable
    :param input_color: color space of source image 
    :param backend: computer vision library which handle the task
    """
    def __init__(self, image=None, input_color=ColorSpace.RGB, backend=CVLib.OPENCV):
        super(GGB, self).__init__(image, backend)
        try:
            assert(isinstance(input_color, ColorSpace))
            self.__img_color_space = input_color
        except:
            raise ColorSpaceError(input_color)


    def process(self, **kwargs):
        """Main process of GGB.
        
        :param kwargs: dict of custom variable
        """
        allowed_kwargs = {'inverse'}
        for k in kwargs:
            if k not in allowed_kwargs:
                raise TypeError('Unexpected keyword argument '
                                'passed to GGB: ' + str(k))
        if self.backend() == CVLib.OPENCV:
            from ggb.backend.opencv_backend import preprocessing, split_normalize, postprocessing
        else:
            from ggb.backend.pil_backend import preprocessing, split_normalize, postprocessing
        img = preprocessing(self.write(), self.__img_color_space)
        b, g = split_normalize(img)
        img = postprocessing(b, g, **kwargs)

        return GGBImage(img, self.backend())



import cv2
import numpy as np

from ggb.utils.constant import ColorSpace


def preprocessing(image, input_color=ColorSpace.RGB):
    """Preprocess source image. 
    
    :param image: source image
    :param input_color: source image color space
    :return:
    """
    img = image
    if input_color == ColorSpace.BGR:
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    elif input_color == ColorSpace.HSV:
        img = cv2.cvtColor(img, cv2.COLOR_HSV2RGB)
    else:
        img = cv2.cvtColor(img, cv2.COLOR_YUV2RGB)
    return img


def split_normalize(image):
    """Splitting source image then normalize it.
    
    :param image: source image
    :return: blue and green channel
    """
    _, g_, b_ = cv2.split(image)
    g_ = cv2.equalizeHist(g_)
    b_ = b_.astype('float32')
    g_ = g_.astype('float32')
    mean = np.mean(b_)
    b_ /= (mean + 1.0)
    mean = np.mean(g_)
    g_ /= (mean + 1.0)
    b_ = np.clip(b_, 0, 1)
    g_ = np.clip(g_, 0, 1)
    b_ *= 255.0
    g_ *= 255.0
    return b_, g_


def postprocessing(b, g, **kwargs):
    """Final GGB process.
    
    :param b: blue channel
    :param g: green channel
    :param kwargs: dict of custom variable
    :return: GGB image
    """
    inverse = False
    if 'inverse' in kwargs:
        inverse = kwargs['inverse']
    img = cv2.merge((b,g,g)) if not inverse else cv2.merge((g,g,b))
    return img



from ggb.utils.constant import ColorSpace, CVLib
from ggb.utils.error import ColorSpaceError

import numpy as np

class GGBImage(object):
    """Image handler for GGB.
    
    :param image: image source either from path or variable
    :param backend: computer vision library which handle the task
    """
    def __init__(self, image=None, backend=CVLib.OPENCV):
        self.__backend = backend
        self.__image = self.__read(image, backend)


    def backend(self):
        """Check which computer vision library is used as backend
        
        :return: type of computer vision library
        """
        return self.__backend


    def __read(self, source, backend=CVLib.OPENCV):
        """Read image from source.
    
        :param source: image source either from path or variable
        :param backend: computer vision library which handle the task
        :return: image variable
        """
        if isinstance(source, str):
            if backend == CVLib.OPENCV:
                self.__backend = CVLib.OPENCV
                import cv2
                return cv2.imread(path)
            else:
                self.__backend = CVLib.PIL
                from PIL import Image
                return Image.open(path).convert('RGB')
        else:
            if isinstance(source, np.ndarray):
                self.__backend = CVLib.OPENCV
            else:
                self.__backend = CVLib.PIL
            return source


    def write(self, path=None, **kwargs):
        """Write the image into a file when path is not None 
           or variable when path is None.
         
        :param path: path to file
        :param kwargs: dict of custom variable
        """
        allowed_kwargs = {'inverse'}
        for k in kwargs:
            if k not in allowed_kwargs:
                raise TypeError('Unexpected keyword argument '
                                'passed to GGBImage: ' + str(k))
        if path == None:
            if self.__backend == CVLib.OPENCV:
                image = self.__image.astype('uint8')
                return image
            return self.__image
        else:
            if self.__backend == CVLib.OPENCV:
                import cv2
                image = self.__image.astype('uint8')
                if 'inverse' in kwargs:
                    image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR) if kwargs['inverse'] else image
                cv2.imwrite(path, image)
            else:
                self.__image.save(path)


    def show(self, **kwargs):
        """Show the image.

        :param kwargs: dict of custom variable
        """
        allowed_kwargs = {'inverse'}
        for k in kwargs:
            if k not in allowed_kwargs:
                raise TypeError('Unexpected keyword argument '
                                'passed to GGBImage: ' + str(k))
        if self.__backend == CVLib.OPENCV:
            import cv2
            image = self.__image.astype('uint8')
            if 'inverse' in kwargs:
                image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR) if kwargs['inverse'] else image
            cv2.imshow("GGB", image)
            cv2.waitKey(0)
        else:
            self.__image.show()



from __future__ import absolute_import


class ColorSpaceError(Exception):
    """Exception raised for errors in the color space.
    
    
    :param color_space: color space which caused the error
    :param message: explanation of the error
    """
    def __init__(self, color_space, message="Color space is not supported"):
        self.color_space = color_space
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.color_space} -> {self.message}'



"""

This package is implementation of GGB color space from 
Development of a Robust Algorithm for Detection of Nuclei and Classification of White Blood Cells in Peripheral Blood Smear Images.
 
https://link.springer.com/content/pdf/10.1007%2Fs10916-018-0962-1.pdf


"""

from __future__ import absolute_import

from ggb.ggb import GGB
from ggb.utils.constant import ColorSpace, CVLib
from ggb.utils.image import GGBImage

__version__ = '1.1.0'



from enum import Enum


class ColorSpace(Enum):
    """Valid color spaces.
    """
    RGB = 1
    BGR = 2
    HSV = 3
    YUV = 4


class CVLib(Enum):
    """Supported computer vision libraries.
    """
    OPENCV = 1
    PIL = 2



from setuptools import setup
from setuptools import find_packages

import pathlib
import os.path

# The directory containing this file
HERE = os.path.abspath(os.path.dirname(__file__))

# The text of the README file
with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(name='ggb',
      version='1.1.0',
      description='GGB Color Space in Python',
      long_description=README,
      long_description_content_type="text/markdown",
      author='Resha Dwika Hefni Al-Fahsi',
      author_email='resha.alfahsi@gmail.com',
      url='https://github.com/reshalfahsi/ggb',
      license='MIT',
      classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
      ],
      include_package_data=True,
      install_requires=['numpy', 'Pillow', 'opencv-python'],
      extras_require={
          'tests': ['pytest',
                    'pytest-cov',
                    'pytest-pep8',
                    'pytest-xdist',
                    'python-coveralls',
                    'coverage==3.7.1'],
      },
      packages=find_packages(),
      entry_points={"console_scripts": ["reshalfahsi=ggb.__main__:main"]},
)



import cv2
import numpy as np

import PIL
from PIL import Image, ImageOps
from PIL.ImageStat import Stat

from ggb.utils.constant import ColorSpace


def preprocessing(image, input_color=ColorSpace.RGB):
    """Preprocess source image. 
    
    :param image: source image
    :param input_color: source image color space
    :return:
    """
    img = image
    if input_color == ColorSpace.BGR:
        img = np.array(img)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        img = Image.fromarray(img)
    elif input_color == ColorSpace.HSV:
        img = np.array(img)
        img = cv2.cvtColor(img, cv2.COLOR_HSV2RGB)
        img = Image.fromarray(img)
    else:
        img = np.array(img)
        img = cv2.cvtColor(img, cv2.COLOR_YUV2RGB)
        img = Image.fromarray(img)
    return img


def split_normalize(image):
    """Splitting source image then normalize it.
    
    :param image: source image
    :return: blue and green channel
    """
    im = Image.Image.split(image)
    g = ImageOps.equalize(im[1], mask = None)
    mean = Stat(im[2]).mean[0]
    b = np.array(im[2]).astype('float32')
    b /= (1.0 + mean) 
    mean = Stat(g).mean[0]
    g = np.array(g).astype('float32')
    g /= (1.0 + mean)
    b = np.clip(b, 0, 1)
    g = np.clip(g, 0, 1)
    b *= 255.0
    g *= 255.0
    b = Image.fromarray(b.astype('uint8'))
    g = Image.fromarray(g.astype('uint8'))
    return b, g


def postprocessing(b, g, **kwargs):
    """Final GGB process.
    
    :param b: blue channel
    :param g: green channel
    :param kwargs: dict of custom variable
    :return: GGB image
    """
    img = Image.merge('RGB', (g, g, b))
    return img
